/*****************************************************************************\
 * AboutDialog.java
 *
 * $Author: blais $
 * $Date: 1997/11/11 23:17:43 $
 *
 * Desc:        About dialog for quadrangulation application.
 *
 *****************************************************************************/
/*****************************************************************************\
 * Copyright (C) 1996, Martin Blais.
 * All Rights Reserved.
 * 
 * This is unpublished proprietary source code of Martin Blais;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Martin Blais.
 *****************************************************************************/

import java.awt.* ;


///////////////////////////////////////////////////////////////////////////////
//
// class AboutDialog
//
///////////////////////////////////////////////////////////////////////////////

public class AboutDialog extends Frame
{
    ///////////////////////////////////////////////////////////////////////////
    // Constants
    
    public final static int xsize = 440 ;
    public final static int ysize = 280 ;

    ///////////////////////////////////////////////////////////////////////////
    // Data members
    
    Button		m_butOk ;
    AboutCanvas 	m_Canvas ;
    Image		m_imgQuaddy ;
    
    ///////////////////////////////////////////////////////////////////////////
    // AboutDialog.AboutDialog
    //
    public AboutDialog( Frame in_Parent, Image imgQuaddy, boolean fModal )
    {
//        super( in_Parent, fModal ) ;
        setTitle( "About" ) ;
        resize( xsize, ysize ) ;
        setResizable( false ) ;
	m_imgQuaddy = imgQuaddy ;

        BuildUI() ;
    }


    ///////////////////////////////////////////////////////////////////////
    // AboutDialog.BuildUI
    //
    public void BuildUI()
    {
        ///////////////////////////////////////////////////////////////////////
        // Build work area
        setLayout( new BorderLayout() ) ;

        // Button panel
        m_butOk = new Button( "Ok" ) ;
        add( "South", m_butOk ) ;

        // Canvas
        m_Canvas = new AboutCanvas( m_imgQuaddy ) ;
        add( "Center", m_Canvas ) ;

        pack() ;
    }


    ///////////////////////////////////////////////////////////////////////////
    // AboutDialog.handleEvent
    // 
    public boolean handleEvent( Event in_Event )
    {
        // Handle window_destroy events
        if ( in_Event.id == Event.WINDOW_DESTROY )
        {
            dispose() ;
	    return true ;
        }
	else
	{
	    return super.handleEvent( in_Event ) ;
	}
    }

    ///////////////////////////////////////////////////////////////////////////
    // AboutDIalog.action
    //
    public boolean action( Event in_Event, Object in_Object )
    {
        boolean fHandled = true ;

        // Handle Ok Button
	if ( in_Object.equals("Ok") )
	{
	    dispose() ;
	}
        return fHandled ;
    }
}



///////////////////////////////////////////////////////////////////////////////
//
// class AboutCanvas
//
///////////////////////////////////////////////////////////////////////////////

class AboutCanvas extends Canvas
{
    protected Image 	m_imgQuaddy ;
    protected int 	m_yy ;

    ///////////////////////////////////////////////////////////////////////////
    // AboutCanvas.AboutCanvas
    //
    AboutCanvas( Image image )
    {
        m_imgQuaddy = image ;
	m_yy = 0 ;
        setBackground( Color.gray.brighter() ) ;
    }

    ///////////////////////////////////////////////////////////////////////////
    // AboutCanvas.paint
    //
    public void paint( Graphics gra )
    {
        reset() ;
        
        // Set text color
        gra.setColor( Color.black ) ;
        
        // Gets some decent fonts
        Font fontBold = new Font( "Helvetica", Font.BOLD, 20) ;
        Font fontItalic = new Font( "Helvetica", Font.ITALIC, 20) ;
        Font fontSmall = new Font( "Dialog", Font.PLAIN, 12) ;
        
        // Write some text
        skip( 50 ) ;
        writeCenteredText( gra, fontBold, "Characterizing and Efficiently Computing" ) ;
        writeCenteredText( gra, fontBold, "Quadrangulations of Planar Point Sets" ) ;
        skip( 20 ) ;
        writeCenteredText( gra, fontItalic, "applet by" ) ;
        writeCenteredText( gra, fontItalic, "Martin Blais" ) ;
        skip( 20 ) ;
        writeCenteredText( gra, fontSmall, "Computational Geometry Project" ) ;
        writeCenteredText( gra, fontSmall, "Course 308-507A - Prof: Godfried Toussaint" ) ;
        skip( 30 ) ;
        writeCenteredText( gra, fontSmall, "Last revision: 11/97" ) ;
        skip( 20 ) ;
	
        Dimension dim = size() ;
        int xx = dim.width ;

        gra.drawImage( m_imgQuaddy, (xx-20)/2, m_yy, 40, 40, this ) ;
    }

    ///////////////////////////////////////////////////////////////////////////
    // AboutCanvas.reset
    //
    private void reset()
    {
	m_yy = 0 ;
    }

    ///////////////////////////////////////////////////////////////////////////
    // AboutCanvas.skip
    //
    private void skip( int pixels )
    {
	m_yy += pixels ;
    }

    ///////////////////////////////////////////////////////////////////////////
    // AboutCanvas.writeCenteredText
    //
    private void writeCenteredText( Graphics gra, Font font, String string )
    {
        Dimension dim = size() ;
        int xx = dim.width ;
        FontMetrics fontMetrics = gra.getFontMetrics( font ) ;
        gra.setFont( font ) ;
        gra.drawString( string, (xx - fontMetrics.stringWidth(string))/2, m_yy ) ;
        
        skip( fontMetrics.getHeight() + 2 ) ;
    }

    ///////////////////////////////////////////////////////////////////////////
    // AboutCanvas.minimumSize
    //
    public Dimension minimumSize()
    {
	return preferredSize() ;
    }

    ///////////////////////////////////////////////////////////////////////////
    // AboutCanvas.preferredSize
    //
    public Dimension preferredSize()
    {
        return new Dimension( AboutDialog.xsize, AboutDialog.ysize ) ;
    }
}
