/*****************************************************************************\
 * Edge.java
 *
 * $Author: blais $
 * $Date: 1997/10/20 01:59:12 $
 *
 * Desc:        Edge class.
 *
 *****************************************************************************/
/*****************************************************************************\
 * Copyright (C) 1996, Martin Blais.
 * All Rights Reserved.
 * 
 * This is unpublished proprietary source code of Martin Blais;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Martin Blais.
 *****************************************************************************/

import Vertex2 ;


///////////////////////////////////////////////////////////////////////////////
//
// Edge
//
///////////////////////////////////////////////////////////////////////////////

class Edge extends Object implements Cloneable
{
    /////////////////////////////////////////////////////////////////////////////
    // Data members

    public Vertex2 v1 ;
    public Vertex2 v2 ;

    ///////////////////////////////////////////////////////////////////////////
    // Constructors
    //
    Edge()
    {
        v1 = null ;
        v2 = null ;
    }

    Edge( Vertex2 nv1, Vertex2 nv2 )
    {  
        v1 = nv1 ;  
        v2 = nv2 ;
    }

    Edge( Edge v )
    {  
        v1 = v.v1 ;  
        v2 = v.v2 ;
    }

    ///////////////////////////////////////////////////////////////////////////
    // Clone
    //
    public Object clone() 
    {
        return new Edge( this ) ;
    }

    ///////////////////////////////////////////////////////////////////////////
    // Assignment
    //
    public void set( Vertex2 nv1, Vertex2 nv2 )
    {  
        v1 = nv1 ;  
        v2 = nv2 ;
    }

    ///////////////////////////////////////////////////////////////////////////
    // Intersection of two line segments (edges)
    //
    public static boolean intersectSegments( Edge e1, Edge e2 )
    {
        try 
        {
            boolean f11 = ( Vertex2.signedArea( e1.v1, e1.v2, e2.v1 ) >= 0 ) ;
            boolean f12 = ( Vertex2.signedArea( e1.v1, e1.v2, e2.v2 ) >= 0 ) ;
            boolean f21 = ( Vertex2.signedArea( e2.v1, e2.v2, e1.v1 ) >= 0 ) ;
            boolean f22 = ( Vertex2.signedArea( e2.v1, e2.v2, e1.v2 ) >= 0 ) ;
            return f11 != f12 && f21 != f22 ;
        }
        catch( VertexException ex )
        {
            System.err.println( "VertexException in intersectSegments" ) ;
            return false ;
        }
    }

}
