/*****************************************************************************\
 * MainCanvas.java
 *
 * $Author: blais $
 * $Date: 1997/11/11 23:17:47 $
 *
 * Desc:        Main Quadulation application class.
 *
 *****************************************************************************/
/*****************************************************************************\
 * Copyright (C) 1996, Martin Blais.
 * All Rights Reserved.
 * 
 * This is unpublished proprietary source code of Martin Blais;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Martin Blais.
 *****************************************************************************/

import java.awt.* ;
import java.io.PrintStream ;
import java.util.Vector ;
import java.util.Enumeration ;

import QuadApp ;

///////////////////////////////////////////////////////////////////////////////
//
// class MainCanvas
//
///////////////////////////////////////////////////////////////////////////////

public class MainCanvas extends MousableCanvas
{
    ///////////////////////////////////////////////////////////////////////////
    // Data members
    private QuadApp 	m_App ;

    // Double-buffering
    private Dimension 	m_offDimension;
    private Image 		m_offImage;
    private Graphics 	m_offGraphics;

    // UI manips
    private int			m_nx ;
    private int			m_ny ;

    // Constants
    final int DIAMETER_POINTS = 6 ;
    final int DIAMETER_SMALLPOINTS = 4 ;
    

    ///////////////////////////////////////////////////////////////////////////
    // MainCanvas.MainCanvas
    // 
    // Constructor
    //
    public MainCanvas( QuadApp in_App )
    {
        super() ;

        m_App = in_App ;

        setBackground( Color.white ) ;

        // REVIEW: 1.1 Only  setCursor( Cursor( CROSSHAIR_CURSOR ) ) ;
    }



    ///////////////////////////////////////////////////////////////////////////
    // MainCanvas.mouseButton1
    //
    public boolean mouseButton1( Event event, int  x, int  y, int mouse )
    {
        boolean fHandled = true ;

        if ( m_App.getState() != QuadApp.ST_ENTERPOINTS )
        {
            return false ;
        }

        switch ( mouse )
        {
            case MOUSE_DOWN:
            {
                // Search for a vertex at this position
                Vertex2 found = m_App.searchVertex( x, y, DIAMETER_POINTS/2 ) ;
                
                // Create a new vertex if not found
                if ( found == null )
                {
                    found = new Vertex2( x, y ) ;
                    Vector rgPoints = m_App.getVertices() ;
                    rgPoints.addElement( found ) ;
                }
                
                // Mark as dragging
                m_App.m_vtxDragging = found ;
                
                repaint() ;
            }
            break ;
            
            case MOUSE_DRAG:
            {
                // Request repaint if a point is being dragged
                if ( m_App.m_vtxDragging != null )
                {
                    m_App.m_vtxDragging.set( x, y ) ;
                    repaint() ;
                }
            }
            break ;
            
            case MOUSE_UP:
            {
                // Add dragging vertex to list of points
                if ( m_App.m_vtxDragging != null )
                {
                    m_App.m_vtxDragging.set( x, y ) ;
                    m_App.m_vtxDragging = null ;
                    repaint() ;
                }
            }
            break ;

            default:
            fHandled = false ;
            break ;
        }

        return fHandled ;
    }

    ///////////////////////////////////////////////////////////////////////////
    // MainCanvas.mouseButton2
    //
    public boolean mouseButton2( Event event, int  x, int  y, int mouse )
    {
        boolean fHandled = true ;

        if ( m_App.getState() != QuadApp.ST_ENTERPOINTS )
        {
            return false ;
        }
        
        switch ( mouse )
        {
            case MOUSE_DOWN:
            {
                // Search for a vertex at this position
                Vertex2 found = m_App.searchVertex( x, y, DIAMETER_POINTS/2 ) ;

                if ( found != null )
                {
                    // Remove the element
                    Vector rgPoints = m_App.getVertices() ;
                    if ( rgPoints.removeElement( found ) == false )
                    {
                        System.err.println( "Internal error!" ) ;
                    }
                }

                repaint() ;
            }
            break ;
            
            case MOUSE_DRAG:
            {
            }
            break ;
            
            case MOUSE_UP:
            {
            }
            break ;

            default:
            fHandled = false ;
            break ;
        }

        return fHandled ;
    }

    ///////////////////////////////////////////////////////////////////////////
    // MainCanvas.mouseButton3
    //
    public boolean mouseButton3( Event event, int  x, int  y, int mouse )
    {
        boolean fHandled = true ;

        if ( m_App.getState() != QuadApp.ST_ENTERPOINTS )
        {
            return false ;
        }

        switch ( mouse )
        {
            case MOUSE_DOWN:
            {
                m_nx = x ;
                m_ny = y ;
            }
            break ;
            
            case MOUSE_DRAG:
            {
            }
            break ;
            
            case MOUSE_UP:
            {
                int xmin = Math.min( m_nx, x ) ;
                int xmax = Math.max( m_nx, x ) ;
                int ymin = Math.min( m_ny, y ) ;
                int ymax = Math.max( m_ny, y ) ;

                // Mark all vertices in specified rectangle for removal
                Vector rgPoints = m_App.getVertices() ;
                Vector rgToRemove = new Vector() ;
                Vertex2 v ;
                for ( Enumeration e = rgPoints.elements() ; e.hasMoreElements() ; ) 
                { 
                    v = (Vertex2) e.nextElement() ;

                    if ( v.x >= xmin && v.x <= xmax &&
                         v.y >= ymin && v.y <= ymax )
                    {
                        rgToRemove.addElement( v ) ;
                    }
                }

                // Remove elements
                for ( Enumeration e = rgToRemove.elements() ; e.hasMoreElements() ; ) 
                { 
                    v = (Vertex2) e.nextElement() ;
                    if ( rgPoints.removeElement( v ) == false )
                    {
                        System.err.println( "Internal error!" ) ;
                    }
                }

                repaint() ;
            }
            break ;

            default:
            fHandled = false ;
            break ;
        }
        
        return fHandled ;
    }


    ///////////////////////////////////////////////////////////////////////////
    // MainCanvas.paint
    //
    public void paint( Graphics graphics )
    {
        update( graphics ) ;
    }

    ///////////////////////////////////////////////////////////////////////////
    // MainCanvas.update
    //
    // Note: We override update to prevent it from clearing the screen
    //
    public void update( Graphics graphics )
    {
        Dimension dim = size();

        // Validate backbuffer
        if ( ( m_offGraphics == null )
           || ( dim.width != m_offDimension.width )
           || ( dim.height != m_offDimension.height ) ) 
        {
            m_offDimension = dim ;
            m_offImage = createImage( dim.width, dim.height ) ;
            m_offGraphics = m_offImage.getGraphics() ;
        }

        // Clear backbuffer
        m_offGraphics.setColor( getBackground() ) ;
        m_offGraphics.fillRect( 0, 0, dim.width, dim.height ) ;
        m_offGraphics.setColor( Color.black ) ;

        // Draw contour frame
        m_offGraphics.drawRect(0, 0, dim.width - 1, dim.height - 1);

        ///////////////////////////////////////////////////////////////////////
        // Draw vertices
        //

        if ( m_App.getState() != QuadApp.ST_FINALQUADRANG )
        {
            // Draw dragged element
            if ( m_App.m_vtxDragging != null )
            {
                drawVertex( m_App.m_vtxDragging, Color.red ) ;
            }

            Vector rgEdges ;

            // Draw convex hull
            drawEdgeVector( m_App.getConvexHull(), Color.blue ) ;
        
            // Draw spiral
            drawEdgeVector( m_App.getSpiral(), Color.red ) ;

            // Draw closing vertex
            drawEdgeVector( m_App.getCloseChain(), Color.pink ) ;

            // Draw Ph
            Vertex2 vPh = m_App.getPh() ;
            if ( vPh != null )
            {
                m_offGraphics.setColor( Color.pink ) ;
                m_offGraphics.drawOval( (int) vPh.x - DIAMETER_POINTS/2 -2,
                                        (int) vPh.y - DIAMETER_POINTS/2 -2, 
                                        DIAMETER_POINTS +4, DIAMETER_POINTS +4 ) ;
            }

            // Draw diagonals for inner star-shaped polygon
            drawEdgeVector( m_App.getInnerDiagonals(), Color.cyan ) ;

            // Draw diagonals for polygonal chain
            drawEdgeVector( m_App.getChainDiagonals(), Color.green ) ;

            // Draw Y
            Vertex2 vY = m_App.getY() ;
            if ( vY != null )
            {
                m_offGraphics.setColor( Color.black ) ;
                m_offGraphics.drawOval( (int) vY.x - DIAMETER_POINTS/2 -2, 
                                        (int) vY.y - DIAMETER_POINTS/2 -2, 
                                        DIAMETER_POINTS +4, DIAMETER_POINTS +4 ) ;
            }

            // Draw Steiner point
            Vertex2 vSteiner = m_App.getSteiner() ;
            if ( vSteiner != null )
            {
                m_offGraphics.setColor( Color.red ) ;
                m_offGraphics.drawOval( (int) vSteiner.x - DIAMETER_POINTS/2 -2, 
                                        (int) vSteiner.y - DIAMETER_POINTS/2 -2, 
                                        DIAMETER_POINTS +4, DIAMETER_POINTS +4 ) ;
            }

            // Draw vertices
            Vector rgPoints = m_App.getVertices() ;
            Vertex2 v ;
            for ( Enumeration e = rgPoints.elements() ; e.hasMoreElements() ; ) 
            { 
                v = (Vertex2) e.nextElement() ;

                drawVertex( v, Color.yellow ) ;
            } 
        }
        else
        {
            // Draw small vertices
            Vector rgPoints = m_App.getVertices() ;
            Vertex2 v ;
            for ( Enumeration e = rgPoints.elements() ; e.hasMoreElements() ; ) 
            { 
                v = (Vertex2) e.nextElement() ;
                m_offGraphics.setColor( Color.black ) ;
                m_offGraphics.fillOval( (int) v.x - DIAMETER_SMALLPOINTS/2, 
                                        (int) v.y - DIAMETER_SMALLPOINTS/2, 
                                        DIAMETER_SMALLPOINTS, DIAMETER_SMALLPOINTS ) ;
            } 

            // Draw only final quadrangulation 
            drawEdgeVector( m_App.getFinalQuadrang(), Color.black ) ;
        }
        
        // Paint the image onto the screen.
        graphics.drawImage( m_offImage, 0, 0, this ) ;
    }

    ///////////////////////////////////////////////////////////////////////////
    // MainCanvas.drawVertex
    // 
    private void drawVertex( Vertex2 v, Color col )
    {
        m_offGraphics.setColor( col ) ;
        m_offGraphics.fillOval( (int) v.x - DIAMETER_POINTS/2, 
                                (int) v.y - DIAMETER_POINTS/2, 
                                DIAMETER_POINTS, DIAMETER_POINTS ) ;
        
        m_offGraphics.setColor( Color.black ) ;
        m_offGraphics.drawOval( (int) v.x - DIAMETER_POINTS/2, 
                                (int) v.y - DIAMETER_POINTS/2, 
                                DIAMETER_POINTS, DIAMETER_POINTS ) ;
    }

    ///////////////////////////////////////////////////////////////////////////
    // MainCanvas.drawEdge
    // 
    private void drawEdge( Edge edge, Color col )
    {
        m_offGraphics.setColor( col ) ;
        m_offGraphics.drawLine( (int) edge.v1.x, (int) edge.v1.y,
                                (int) edge.v2.x, (int) edge.v2.y ) ;
    }

    ///////////////////////////////////////////////////////////////////////////
    // MainCanvas.drawEdgeVector
    //
    private void drawEdgeVector( Vector vec, Color col )
    {
        Edge edge ;
        for ( Enumeration e2 = vec.elements() ; e2.hasMoreElements() ; ) 
        { 
            edge = (Edge) e2.nextElement() ;
            drawEdge( edge, col ) ;
        }
    }

}
