/*****************************************************************************\
 * MousableCanvas.java
 *
 * $Author: blais $
 * $Date: 1997/10/13 20:15:02 $
 *
 * Desc:        Main Quadulation application class.
 *
 *****************************************************************************/
/*****************************************************************************\
 * Copyright (C) 1996, Martin Blais.
 * All Rights Reserved.
 * 
 * This is unpublished proprietary source code of Martin Blais;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Martin Blais.
 *****************************************************************************/

import java.awt.* ;
import java.io.PrintStream ;
import java.util.Vector ;
import java.util.Enumeration ;

import QuadApp ;

///////////////////////////////////////////////////////////////////////////////
//
// class MousableCanvas
//
///////////////////////////////////////////////////////////////////////////////

public class MousableCanvas extends Canvas
{
    ///////////////////////////////////////////////////////////////////////////
    // Constants
    public final static int MOUSE_DOWN = 1 ;
    public final static int MOUSE_DRAG = 2 ;
    public final static int MOUSE_UP = 3 ;

    public final static int NOBUTTON = 0 ;
    public final static int BUTTON1 = 1 ;
    public final static int BUTTON2 = 2 ;
    public final static int BUTTON3 = 3 ;

    ///////////////////////////////////////////////////////////////////////////
    // MousableCanvas.MousableCanvas
    // 
    // Constructor
    //
    public MousableCanvas()
    {
        super() ;
    }

    ///////////////////////////////////////////////////////////////////////////
    // MousableCanvas.mouseDown
    //
    public boolean mouseDown( Event event, int  x, int  y )
    {
        boolean fHandled1 ;
        boolean fHandled2 ;
        int button = whichButton( event ) ;

        switch ( button )
        {
            case BUTTON1:
            fHandled1 = mouseDown( event, x, y, BUTTON1 ) ;
            fHandled2 = mouseButton1( event, x, y, MOUSE_DOWN ) ;
            break ;

            case BUTTON2:
            fHandled1 = mouseDown( event, x, y, BUTTON2 ) ;
            fHandled2 = mouseButton2( event, x, y, MOUSE_DOWN ) ;
            break ;

            case BUTTON3:
            fHandled1 = mouseDown( event, x, y, BUTTON3 ) ;
            fHandled2 = mouseButton3( event, x, y, MOUSE_DOWN ) ;
            break ;

            default:
            fHandled1 = false ;
            fHandled2 = false ;
            break ;
        }

        return fHandled1 || fHandled2 ;
    }

    ///////////////////////////////////////////////////////////////////////////
    // MousableCanvas.mouseDrag
    //
    public boolean mouseDrag( Event event, int  x, int  y )
    {
        boolean fHandled1 ;
        boolean fHandled2 ;
        int button = whichButton( event ) ;

        switch ( button )
        {
            case BUTTON1:
            fHandled1 = mouseDrag( event, x, y, BUTTON1 ) ;
            fHandled2 = mouseButton1( event, x, y, MOUSE_DRAG ) ;
            break ;

            case BUTTON2:
            fHandled1 = mouseDrag( event, x, y, BUTTON2 ) ;
            fHandled2 = mouseButton2( event, x, y, MOUSE_DRAG ) ;
            break ;

            case BUTTON3:
            fHandled1 = mouseDrag( event, x, y, BUTTON3 ) ;
            fHandled2 = mouseButton3( event, x, y, MOUSE_DRAG ) ;
            break ;

            default:
            fHandled1 = false ;
            fHandled2 = false ;
            break ;
        }

        return fHandled1 || fHandled2 ;
    }

    ///////////////////////////////////////////////////////////////////////////
    // MousableCanvas.mouseUp
    //
    public boolean mouseUp( Event event, int  x, int  y )
    {
        boolean fHandled1 ;
        boolean fHandled2 ;
        int button = whichButton( event ) ;

        switch ( button )
        {
            case BUTTON1:
            fHandled1 = mouseUp( event, x, y, BUTTON1 ) ;
            fHandled2 = mouseButton1( event, x, y, MOUSE_UP ) ;
            break ;

            case BUTTON2:
            fHandled1 = mouseUp( event, x, y, BUTTON2 ) ;
            fHandled2 = mouseButton2( event, x, y, MOUSE_UP ) ;
            break ;

            case BUTTON3:
            fHandled1 = mouseUp( event, x, y, BUTTON3 ) ;
            fHandled2 = mouseButton3( event, x, y, MOUSE_UP ) ;
            break ;

            default:
            fHandled1 = false ;
            fHandled2 = false ;
            break ;
        }

        return fHandled1 || fHandled2 ;
    }


    ///////////////////////////////////////////////////////////////////////////
    // MousableCanvas.whichButton
    //
    public int whichButton( Event event )
    {
        int button ;

        if ( ( event.modifiers & Event.ALT_MASK ) == 0 &&  // Button 1
             ( event.modifiers & Event.META_MASK ) == 0 )
        {
            return BUTTON1 ;
        }
        else if ( ( event.modifiers & Event.ALT_MASK ) != 0 &&  // Button 2
                  ( event.modifiers & Event.META_MASK ) == 0 )
        {
            return BUTTON2 ;
        }
        else if ( ( event.modifiers & Event.ALT_MASK ) == 0 &&  // Button 3
                  ( event.modifiers & Event.META_MASK ) != 0 )
        {
            return BUTTON3 ;
        }

        return NOBUTTON ;
    }


    ///////////////////////////////////////////////////////////////////////////
    // MousableCanvas.mouseDown
    //
    public boolean mouseDown( Event event, int  x, int  y, int button )
    {
        return false ;
    }

    ///////////////////////////////////////////////////////////////////////////
    // MousableCanvas.mouseDrag
    //
    public boolean mouseDrag( Event event, int  x, int  y, int button )
    {
        return false ;
    }

    ///////////////////////////////////////////////////////////////////////////
    // MousableCanvas.mouseUp
    //
    public boolean mouseUp( Event event, int  x, int  y, int button )
    {
        return false ;
    }

    ///////////////////////////////////////////////////////////////////////////
    // MousableCanvas.mouseButton1
    //
    public boolean mouseButton1( Event event, int  x, int  y, int mouse )
    {
        return false ;
    }

    ///////////////////////////////////////////////////////////////////////////
    // MousableCanvas.mouseButton2
    //
    public boolean mouseButton2( Event event, int  x, int  y, int mouse )
    {
        return false ;
    }

    ///////////////////////////////////////////////////////////////////////////
    // MousableCanvas.mouseButton3
    //
    public boolean mouseButton3( Event event, int  x, int  y, int mouse )
    {
        return false ;
    }
}
