/*****************************************************************************\
 * QuadApp.java
 *
 * $Author: blais $
 * $Date: 1997/11/14 02:21:26 $
 *
 * Desc:        Main Quadulation application class.
 *
 *****************************************************************************/
/*****************************************************************************\
 * Copyright (C) 1996, Martin Blais.
 * All Rights Reserved.
 * 
 * This is unpublished proprietary source code of Martin Blais;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Martin Blais.
 *****************************************************************************/

import java.awt.* ;
import java.io.PrintStream ;
import java.util.Vector ;
import java.util.Enumeration ;
import java.lang.Math ;
import java.util.Random ;
import java.lang.Exception ;
import java.lang.Integer ;
import java.net.URL ;

import AboutDialog ;
import ManualDialog ;
import MainCanvas ;
import Vertex2 ;
import Edge ;



///////////////////////////////////////////////////////////////////////////////
//
// class QuadApp
//
///////////////////////////////////////////////////////////////////////////////

public class QuadApp extends Frame
{
    ///////////////////////////////////////////////////////////////////////////
    // Data members
    protected boolean		m_fInAnApplet ;
    
    // GUI
    private Button			m_butClear ;
    private Button			m_butNext ;
    private Button			m_butStop ;
    private MainCanvas		m_Canvas ;
    private Label			m_EchoArea ;

    // Core
    private Vector			m_rgVertices ;
    private Vector			m_rgCH ;

    private Vertex2			m_vminx ;
    private Vector			m_rgSpiral ;
    private Vector			m_rgCloseChain ;
    private Vertex2			m_vPh ;
    private Vector			m_rgInnerDiagonals ; // From inside (center) towards outside
    private Vertex2			m_vY ;
    private Vector			m_rgChainDiagonals ; // From outside towards inside (center)
    private Vertex2			m_vSteiner ;
    private Vector			m_rgFinalQuadrang ;
    public Vertex2			m_vtxDragging ;
    private Image			m_imgQuaddy ;
    private URL				m_codeBase ;

    AboutDialog 			m_dlgAbout ;
    ManualDialog 			m_dlgManual ;

    // Incremental computation state
    private int						m_enState ;
    public final static int 		ST_ENTERPOINTS 		= 0 ;
    public final static int 		ST_GENERALPOSITION	= 1 ;
    public final static int 		ST_SPIRAL 			= 2 ;
    public final static int 		ST_CLOSECHAIN 		= 3 ;
    public final static int 		ST_INNERREGION 		= 4 ;
    public final static int 		ST_TRIANGULATE 		= 5 ;
    public final static int 		ST_REMOVEDIAGONALS 	= 6 ;
    public final static int 		ST_STEINERPOINT		= 7 ;
    public final static int 		ST_FINALQUADRANG	= 8 ;

    ///////////////////////////////////////////////////////////////////////////
    // QuadApp.QuadApp
    // 
    // Constructor
    //
    public QuadApp( Image imgQuaddy, URL codeBase )
    {
        m_fInAnApplet = true ;
            
        setTitle( "Quadulation Application" ) ;
        //	setIconImage( Image ) ;

        ///////////////////////////////////////////////////////////////////////
        // Build UI
        BuildUI() ;

        m_rgVertices = new Vector() ;
        m_rgCH = new Vector() ;

        m_vminx = null ;
        m_rgSpiral = new Vector() ;
        m_rgCloseChain = new Vector() ;
        m_vPh = null ; 
        m_rgInnerDiagonals = new Vector() ;
        m_vY = null ; 
        m_rgChainDiagonals = new Vector() ;
        m_vSteiner = null ;
        m_rgFinalQuadrang = new Vector() ;

	m_imgQuaddy = imgQuaddy ;
	m_codeBase = codeBase ;

	m_dlgAbout = null ;
	m_dlgManual = null ;

        changeState( ST_ENTERPOINTS ) ;
    }

    ///////////////////////////////////////////////////////////////////////
    // Build UI
    //
    public void BuildUI()
    {
        ///////////////////////////////////////////////////////////////////////
        // Build menus
        MenuBar menuBar = new MenuBar() ;
        setMenuBar( menuBar ) ;
        
        // Menu "File"
        Menu menuFile = new Menu( "File" ) ;
        menuBar.add( menuFile ) ;
            
        menuFile.add( new MenuItem( "Clear" ) ) ;
        menuFile.add( new MenuItem( "Clear Edges" ) ) ;
        menuFile.add( new MenuItem( "-" ) ) ;
        menuFile.add( new MenuItem( "Exit" ) ) ;
            
        // Menu "Points"
        Menu menuPoints = new Menu( "Points" ) ;
        menuBar.add( menuPoints ) ;

        menuPoints.add( new MenuItem( "Generate 25 random points (Uniform)" ) ) ;
        menuPoints.add( new MenuItem( "Generate 100 random points (Uniform)" ) ) ;
        menuPoints.add( new MenuItem( "Generate 25 random points (Gaussian)" ) ) ;
        menuPoints.add( new MenuItem( "Generate 100 random points (Gaussian)" ) ) ;
        menuPoints.add( new MenuItem( "Generate 25 random points (Poisson)" ) ) ;
        menuPoints.add( new MenuItem( "Generate 100 random points (Poisson)" ) ) ;

        // Menu "Functions"
        Menu menuFuncs = new Menu( "Functions" ) ;
        menuBar.add( menuFuncs ) ;

        menuFuncs.add( new MenuItem( "Compute Convex Hull in O(n log n)" ) ) ;
        menuFuncs.add( new MenuItem( "Compute Spiral in O(n^2)" ) ) ;

        // Menu "Help"
        Menu menuHelp = new Menu( "Help" ) ;
        menuBar.add( menuHelp ) ;
        menuBar.setHelpMenu( menuHelp ) ;

        menuHelp.add( new MenuItem( "Manual" ) ) ;
        menuHelp.add( new MenuItem( "-" ) ) ;
        menuHelp.add( new MenuItem( "About" ) ) ;

        ///////////////////////////////////////////////////////////////////////
        // Build work area
        setLayout( new BorderLayout() ) ;

        // Button panel
        m_butClear = new Button( "Clear" ) ;
        m_butNext = new Button( "Start" ) ;
        m_butStop = new Button( "Stop" ) ;

        Panel buttonPanel = new Panel() ;
        buttonPanel.setLayout( new GridLayout( 3, 1 ) ) ;
        buttonPanel.add( m_butClear ) ;
        buttonPanel.add( m_butNext ) ;
        buttonPanel.add( m_butStop ) ;
        add( "West", buttonPanel ) ;

        // Canvas
        m_Canvas = new MainCanvas( this ) ;
        add( "Center", m_Canvas ) ;

        // Echo area
        m_EchoArea = new Label( "", Label.CENTER ) ;
        add( "South", m_EchoArea ) ;
        // Change echo area's font
        Font fontBig1 = new Font( "Helvetica", Font.BOLD, 16 ) ;
        Font fontBig2 = new Font( "helvetica", Font.BOLD, 16 ) ;
        if ( fontBig1 != null )
            m_EchoArea.setFont( fontBig1 ) ;
        else if ( fontBig2 != null )
            m_EchoArea.setFont( fontBig2 ) ;
    }

    ///////////////////////////////////////////////////////////////////////////
    // QuadApp.handleEvent
    // 
    public boolean handleEvent( Event in_Event )
    {
        // Handle window_destroy events
        if ( in_Event.id == Event.WINDOW_DESTROY )
        {
            terminate() ;
        }
        return super.handleEvent( in_Event ) ;
    }

    ///////////////////////////////////////////////////////////////////////////
    // Quadulation.action
    //
    public boolean action( Event in_Event, Object in_Object )
    {
        boolean fHandled = true ;
        
        // Handle menu items
        if ( in_Event.target instanceof MenuItem )
        {
            if ( in_Object.equals( "Clear" ) )
            {
                clear() ;
                // Reset state
                changeState( ST_ENTERPOINTS ) ;
            }
            if ( in_Object.equals( "Clear Edges" ) )
            {
                clearEdges() ;
                // Reset state
                changeState( ST_ENTERPOINTS ) ;
            }
            else if ( in_Object.equals( "Exit" ) )
            {
                terminate() ;
            }
            else if ( in_Object.equals( "Generate 25 random points (Uniform)" ) )
            {
                generateUniformPoints( 25 ) ;
                m_Canvas.repaint() ;
            }
            else if ( in_Object.equals( "Generate 100 random points (Uniform)" ) )
            {
                generateUniformPoints( 100 ) ;
                m_Canvas.repaint() ;
            }
            else if ( in_Object.equals( "Generate 25 random points (Gaussian)" ) )
            {
                generateGaussianPoints( 25 ) ;
                m_Canvas.repaint() ;
            }
            else if ( in_Object.equals( "Generate 100 random points (Gaussian)" ) )
            {
                generateGaussianPoints( 100 ) ;
                m_Canvas.repaint() ;
            }
            else if ( in_Object.equals( "Generate 25 random points (Poisson)" ) )
            {
                generatePoissonPoints( 25 ) ;
                m_Canvas.repaint() ;
            }
            else if ( in_Object.equals( "Generate 100 random points (Poisson)" ) )
            {
                generatePoissonPoints( 100 ) ;
                m_Canvas.repaint() ;
            }
            else if ( in_Object.equals( "Manual" ) )
            {
                getManualDialog().show() ;
            }
            else if ( in_Object.equals( "About" ) )
            {
                // Note: The modal parameter works around a bug in JDK-1.0
                getAboutDialog().show() ;
		
            }
            else if ( in_Object.equals( "Compute Convex Hull in O(n log n)" ) )
            {
                try 
                {
                    computeConvexHullOnlogn() ;
                    m_Canvas.repaint() ;
                }
                catch ( QuadInternalErrorException ex )
                {
                    System.err.println( "Internal exception occured" ) ;
                    System.exit( 1 ) ;
                }
                catch ( VertexException ex )
                {
                    System.err.println( "Vertex exception" ) ;
                }
            }
            else if ( in_Object.equals( "Compute Spiral in O(n^2)" ) )
            {
                try 
                {
                    // Clear the old spiral
                    m_rgSpiral.removeAllElements() ;

                    validateGeneralPosition( m_rgVertices ) ;

                    computeSpiralOn2() ;
                    m_Canvas.repaint() ;
                }
                catch ( QuadInternalErrorException ex )
                {
                    System.err.println( "Internal exception occured" ) ;
                    System.exit( 1 ) ;
                }
            }
            else
            {
                fHandled = false ;
            }
        }

        // Buttons toolbar
        if ( in_Event.target instanceof Button )
        {
            if ( in_Event.target == m_butClear )
            {
                clear() ;
                // Reset state
                changeState( ST_ENTERPOINTS ) ;
            }
            else if ( in_Event.target == m_butNext )
            {
                switch ( m_enState )
                {
                    case ST_ENTERPOINTS:
                    {
                        changeState( ST_GENERALPOSITION ) ;
                        clearEdges() ;
                        validateGeneralPosition( m_rgVertices ) ;
                    }
                    break ;
                    
                    case ST_GENERALPOSITION:
                    {
                        changeState( ST_SPIRAL ) ;
                        try 
                        {
                            computeSpiralOn2() ;
                            m_Canvas.repaint() ;
                        }
                        catch ( QuadInternalErrorException ex )
                        {
                            System.err.println( "Internal exception occured on computing spiral" ) ;
                        }
                        m_Canvas.repaint() ;
                    }
                    break ;

                    case ST_SPIRAL:
                    {
                        changeState( ST_CLOSECHAIN ) ;
                        try 
                        {
                            computeCloseChain() ;
                            m_Canvas.repaint() ;
                        }
                        catch ( QuadInternalErrorException ex )
                        {
                            System.err.println( "Internal exception occured on computing closing vertex" ) ;
                        }
                        m_Canvas.repaint() ;
                    }
                    break ;

                    case ST_CLOSECHAIN:
                    {
                        changeState( ST_INNERREGION ) ;
                        try 
                        {
                            computeInnerDiagonals() ;
                            m_Canvas.repaint() ;
                        }
                        catch ( QuadInternalErrorException ex )
                        {
                            System.err.println( "Internal exception occured on computing inner region" ) ;
                        }
                        catch ( VertexException ex )
                        {
                            System.err.println( "Vertex exception" ) ;
                        }
                        m_Canvas.repaint() ;
                    }
                    break ;

                    case ST_INNERREGION:
                    {
                        changeState( ST_TRIANGULATE ) ;
                        try 
                        {
                            computeChainDiagonals() ;
                            m_Canvas.repaint() ;
                        }
                        catch ( QuadInternalErrorException ex )
                        {
                            System.err.println( "Internal exception occured on polygonal chain triangulation" ) ;
                        }
                        m_Canvas.repaint() ;
                    }
                    break ;
                    
                    case ST_TRIANGULATE:
                    {
                        changeState( ST_REMOVEDIAGONALS ) ;
                        try 
                        {
                            computeRemoveDiagonals() ;
                            m_Canvas.repaint() ;
                        }
                        catch ( QuadInternalErrorException ex )
                        {
                            System.err.println( "Internal exception occured on removing diagonals" ) ;
                        }
                        m_Canvas.repaint() ;
                    }
                    break ;
                    
                    case ST_REMOVEDIAGONALS:
                    {
                        changeState( ST_STEINERPOINT ) ;
                        try 
                        {
                            computeSteiner() ;
                            m_Canvas.repaint() ;
                        }
                        catch ( QuadInternalErrorException ex )
                        {
                            System.err.println( "Internal exception occured on computing Steiner point" ) ;
                        }
                        m_Canvas.repaint() ;
                    }
                    break ;
                    
                    case ST_STEINERPOINT:
                    {
                        changeState( ST_FINALQUADRANG ) ;
                        try 
                        {
                            computeFinalQuadrang() ;
                            m_Canvas.repaint() ;
                        }
                        catch ( QuadInternalErrorException ex )
                        {
                            System.err.println( "Internal exception occured on computing final quadrangulation" ) ;
                        }
                        m_Canvas.repaint() ;
                    }
                    break ;

                    case ST_FINALQUADRANG:
                    {
                        changeState( ST_ENTERPOINTS ) ;
                        clearEdges() ;
                        m_Canvas.repaint() ;
                    }
                    break ;
                }
            }
            else if ( in_Event.target == m_butStop )
            {
                changeState( ST_ENTERPOINTS ) ;
            }
        }
        return fHandled ;
    }

    ///////////////////////////////////////////////////////////////////////////
    // QuadApp.getState
    //
    public int getState()
    {
        return m_enState ;
    }

    ///////////////////////////////////////////////////////////////////////////
    // QuadApp.changeState
    //
    public void changeState( int newState )
    {
        m_enState = newState ;
        switch ( newState )
        {
            case ST_ENTERPOINTS:
            {
                m_EchoArea.setText( "Select or generate some points and then click \"start\" to quadrangulate." ) ;
                m_butNext.setLabel( "Start" ) ;
                m_butStop.disable() ;
            }
            break ;

            case ST_GENERALPOSITION:
            {
                m_EchoArea.setText( "Validating vertices for general position." ) ;
                m_butNext.setLabel( "Next" ) ;
                m_butStop.enable() ;
            }
            break ;

            case ST_SPIRAL:
            {
                m_EchoArea.setText( "Computing spiral using rotating calipers." ) ;
                m_butNext.setLabel( "Next" ) ;
                m_butStop.enable() ;
            }
            break ;

            case ST_CLOSECHAIN:
            {
                m_EchoArea.setText( "Closing polygonal chain." ) ;
                m_butNext.setLabel( "Next" ) ;
                m_butStop.enable() ;
            }
            break ;
            
            case ST_INNERREGION:
            {
                m_EchoArea.setText( "Determining and triangulating inner star-shaped region." ) ;
                m_butNext.setLabel( "Next" ) ;
                m_butStop.enable() ;
            }
            break ;

            case ST_TRIANGULATE:
            {
                m_EchoArea.setText( "Triangulating spiral." ) ;
                m_butNext.setLabel( "Next" ) ;
                m_butStop.enable() ;
            }
            break ;

            case ST_REMOVEDIAGONALS:
            {
                m_EchoArea.setText( "Removing diagonals of triangulation to quadrangulate." ) ;
                m_butNext.setLabel( "Next" ) ;
                m_butStop.enable() ;
            }
            break ;

            case ST_STEINERPOINT:
            {
                m_EchoArea.setText( "Optionally adding Steiner point to complete last quadrangle." ) ;
                m_butNext.setLabel( "Next" ) ;
                m_butStop.enable() ;
            }
            break ;

            case ST_FINALQUADRANG:
            {
                m_EchoArea.setText( "Final quadrangulation." ) ;
                m_butNext.setLabel( "End" ) ;
                m_butStop.disable() ;
            }
            break ;
        }
    }

    ///////////////////////////////////////////////////////////////////////////
    // Accessors
    //
    public Vector getVertices() 		{ return m_rgVertices ; }
    public Vector getConvexHull() 		{ return m_rgCH ; }

    public Vector getSpiral() 			{ return m_rgSpiral ; }
    public Vector getCloseChain() 		{ return m_rgCloseChain ; }
    public Vertex2 getPh() 				{ return m_vPh ; } 
    public Vector getInnerDiagonals() 	{ return m_rgInnerDiagonals ; }
    public Vertex2 getY() 				{ return m_vY ; } 
    public Vector getChainDiagonals() 	{ return m_rgChainDiagonals ; }
    public Vertex2 getSteiner() 			{ return m_vSteiner ; }
    public Vector getFinalQuadrang()	{ return m_rgFinalQuadrang ; }

    ///////////////////////////////////////////////////////////////////////////
    // QuadApp.clear
    //
    public void clear()
    {
        m_rgVertices.removeAllElements() ;
        m_rgCH.removeAllElements() ;

        clearEdges() ;

        m_Canvas.repaint() ;
    }

    ///////////////////////////////////////////////////////////////////////////
    // QuadApp.clearEdges
    //
    public void clearEdges()
    {
        m_vminx = null ;
        m_rgSpiral.removeAllElements() ;
        m_rgCloseChain.removeAllElements() ;
        m_vPh = null ;
        m_rgInnerDiagonals.removeAllElements() ;
        m_vY = null ;
        m_rgChainDiagonals.removeAllElements() ;
        m_vSteiner = null ;

        m_Canvas.repaint() ;
    }

    ///////////////////////////////////////////////////////////////////////////
    // QuadApp.generateUniformPoints
    //
    public void generateUniformPoints( int n )
    {
        // Cluster points somewhere towards the center of the canvas
        int BORDER = 100 ;
        Dimension dim = m_Canvas.size() ;
        dim.width = Math.max( 0, dim.width - BORDER * 2 ) ;
        dim.height = Math.max( 0, dim.height - BORDER * 2 ) ;

        Random ran =  new Random() ;
        // Generate new points
        for ( int ii = 0 ; ii < n ; ii++ )
        {
            int x = (int) ( ran.nextDouble() * dim.width ) + BORDER ;
            int y = (int) ( ran.nextDouble() * dim.height ) + BORDER ;
            m_rgVertices.addElement( new Vertex2( x, y ) ) ;
        }
    }

    ///////////////////////////////////////////////////////////////////////////
    // QuadApp.generateGaussianPoints
    //
    public void generateGaussianPoints( int n )
    {
        // Cluster points somewhere towards the center of the canvas
        Dimension dim = m_Canvas.size() ;

        Random ran = new Random() ;
        // Generate new points
        for ( int ii = 0 ; ii < n ; ii++ )
        {
            int x = (int) ( ran.nextGaussian() * dim.width * 0.15 ) + dim.width / 2 ;
            int y = (int) ( ran.nextGaussian() * dim.height * 0.15 ) + dim.height / 2 ; 
            x = Math.min( Math.max( 10, x ), dim.width - 10 ) ; // Avoid points on the border
            y = Math.min( Math.max( 10, y ), dim.height - 10 ) ; // Avoid points on the border
            
            m_rgVertices.addElement( new Vertex2( x, y ) ) ;
        }
    }

    ///////////////////////////////////////////////////////////////////////////
    // QuadApp.generatePoissonPoints
    //
    public void generatePoissonPoints( int n )
    {
        // Cluster points somewhere towards the center of the canvas
        Dimension dim = m_Canvas.size() ;

        Random ran = new Random() ;
        // Generate new points
        for ( int ii = 0 ; ii < n ; ii++ )
        {
            int x = (int) ( ran.nextGaussian() * dim.width * 0.15 ) + dim.width / 2 ;
            int y = (int) ( ran.nextGaussian() * dim.height * 0.15 ) + dim.height / 2 ; 
            x = Math.min( Math.max( 10, x ), dim.width - 10 ) ; // Avoid points on the border
            y = Math.min( Math.max( 10, y ), dim.height - 10 ) ; // Avoid points on the border
            
            m_rgVertices.addElement( new Vertex2( x, y ) ) ;
        }
    }

    ///////////////////////////////////////////////////////////////////////////
    // QuadApp.searchVertex
    // 
    // Searches for a vertex in the list with a certain tolerance (box around
    // vertex).
    public Vertex2 searchVertex( int x, int y, int tolerance )
    {
        // Search for a point with the current position
        Vertex2 v ;
        for ( Enumeration e = m_rgVertices.elements() ; e.hasMoreElements() ; ) 
        { 
            v = (Vertex2) e.nextElement() ;
            
            if ( v.x >= x - tolerance && v.x <= x + tolerance &&
                 v.y >= y - tolerance && v.y <= y + tolerance )
            {
                //System.out.println( "Found vertex!" ) ;
                return v ;
            }
        }

        return null ;
    }

    ///////////////////////////////////////////////////////////////////////////
    // QuadApp.quicksortVertices
    // 
    // Sorts the vertices according to their x coordinate in O(n log n)
    //
    public void quicksortVertices( Vector A, int l, int r )
    {
        Vertex2 temp ;

        if ( l < r )
        {
            int i = l ; // i scans from the left to find elements >= the pivot
            int j = r+1 ; // j scans from the right to find elements <= the pivot
            Vertex2 v = (Vertex2) A.elementAt( l ) ; // v is the pivot element
            
            while ( i < j )
            {
                i++ ;
                while ( i <= r && ((Vertex2)A.elementAt( i )).x < v.x )
                    i++ ;

                j-- ;
                while ( j >= l && ((Vertex2)A.elementAt( j )).x > v.x )
                    j-- ;

                if ( i <= r ) 
                {
                    temp = ((Vertex2)A.elementAt( i )) ;
                    A.setElementAt( ((Vertex2)A.elementAt( j )), i ) ;
                    A.setElementAt( temp, j ) ;
                }
            }
            
            if ( i <= r ) // Undo extra swap
            {
                temp = ((Vertex2)A.elementAt( i )) ;
                A.setElementAt( ((Vertex2)A.elementAt( j )), i ) ;
                A.setElementAt( temp, j ) ;
            }

            // Move the pivot element into its proper position
            temp = ((Vertex2)A.elementAt( j )) ;
            A.setElementAt( ((Vertex2)A.elementAt( l )), j ) ;
            A.setElementAt( temp, l ) ;

            quicksortVertices( A, l, j-1 ) ;
            quicksortVertices( A, j+1, r ) ;
        }
    }

    ///////////////////////////////////////////////////////////////////////////
    // QuadApp.outputVertices
    // 
    public void outputVertices()
    {
        for ( Enumeration e = m_rgVertices.elements() ; e.hasMoreElements() ; ) 
        { 
            Vertex2 v = (Vertex2) e.nextElement() ;
            System.out.println( " Vertex: " + v.x + "  " + v.y ) ;
        }
    }

    ///////////////////////////////////////////////////////////////////////////
    // QuadApp.validateGeneralPosition
    // 
    // Deletes copunctual vertices
    // Note: We don't care so much about collinear vertices for now.
    // Note2: VERTICES HAVE TO BE SORTED FOR THIS TO WORK.
    //
    public void validateGeneralPosition( Vector rgVertices )
    {
        for ( int ii = 0 ; ii < rgVertices.size()-1 ; ii++ ) 
        { 
            Vertex2 v1 = (Vertex2) rgVertices.elementAt( ii ) ;
            Vertex2 v2 = (Vertex2) rgVertices.elementAt( ii+1 ) ;
            if ( v1.equals( v2 ) )
            {
                rgVertices.removeElementAt( ii ) ;
                ii-- ;
            }
        }
    }


    ///////////////////////////////////////////////////////////////////////////
    // Compute convex hull in O(nlogn)
    //
    public void computeConvexHullOnlogn() 
        throws QuadInternalErrorException, VertexException
    {
        // Clear the old convex hull
        m_rgCH.removeAllElements() ;

        if ( m_rgVertices.size() <= 2 )
            return ;

        Vector rgCHVertices = new Vector() ; // Vertices of the temporary convex hull

        // Sort the vertices in x-coordinate
        quicksortVertices( m_rgVertices, 0, m_rgVertices.size()-1 ) ;
        validateGeneralPosition( m_rgVertices ) ;

        ///////////////////////////////////////////////////////////////////////
        // Higher hull

        // Initialize algorithm with first edge (from first two vertices)
        // Add the first vertex in x to the list
        Enumeration e = m_rgVertices.elements() ;
        rgCHVertices.addElement( e.nextElement() ) ;
        rgCHVertices.addElement( e.nextElement() ) ;

        // Until we get to the max X vertex
        while ( e.hasMoreElements() )
        {             
            // Add the next vertex in x to the list
            rgCHVertices.addElement( e.nextElement() ) ;
            
            // See if we can't remove some of the previous vertices
            while ( rgCHVertices.size() >= 3 &&
                    Vertex2.signedArea( (Vertex2) rgCHVertices.elementAt( rgCHVertices.size()-1 ), 
                                        (Vertex2) rgCHVertices.elementAt( rgCHVertices.size()-2 ), 
                                        (Vertex2) rgCHVertices.elementAt( rgCHVertices.size()-3 ) ) >= 0 )
            {
                rgCHVertices.removeElementAt( rgCHVertices.size()-2 ) ;
            }
        }

        ///////////////////////////////////////////////////////////////////////
        // Add upper vertices to convex hull

        if ( rgCHVertices.size() < 2 )
            throw new QuadInternalErrorException() ;
        
        e = rgCHVertices.elements() ;
        Vertex2 v1 = (Vertex2) e.nextElement() ;
        Vertex2 v2 ;
        while ( e.hasMoreElements() ) 
        { 
            v2 = (Vertex2) e.nextElement() ;
            
            m_rgCH.addElement( new Edge( v1, v2 ) ) ;

            v1 = v2 ;
        }

        ///////////////////////////////////////////////////////////////////////
        // Lower hull

        // Initialize algorithm with first edge (from first two vertices)
        // Add the first vertex in x to the list
        rgCHVertices.removeAllElements() ;
        e = m_rgVertices.elements() ;
        rgCHVertices.addElement( e.nextElement() ) ;
        rgCHVertices.addElement( e.nextElement() ) ;

        // Until we get to the max X vertex
        while ( e.hasMoreElements() )
        {             
            // Add the next vertex in x to the list
            rgCHVertices.addElement( e.nextElement() ) ;
            
            // See if we can't remove some of the previous vertices
            while ( rgCHVertices.size() >= 3 &&
                    Vertex2.signedArea( (Vertex2) rgCHVertices.elementAt( rgCHVertices.size()-1 ), 
                                        (Vertex2) rgCHVertices.elementAt( rgCHVertices.size()-2 ), 
                                        (Vertex2) rgCHVertices.elementAt( rgCHVertices.size()-3 ) ) <= 0 )
            {
                rgCHVertices.removeElementAt( rgCHVertices.size()-2 ) ;
            }
        }

        ///////////////////////////////////////////////////////////////////////
        // Add upper vertices to convex hull

        if ( rgCHVertices.size() < 2 )
            throw new QuadInternalErrorException() ;
        
        e = rgCHVertices.elements() ;
        v1 = (Vertex2) e.nextElement() ;
        while ( e.hasMoreElements() ) 
        { 
            v2 = (Vertex2) e.nextElement() ;
            
            m_rgCH.addElement( new Edge( v1, v2 ) ) ;

            v1 = v2 ;
        }
    }

    ///////////////////////////////////////////////////////////////////////////
    //
    // computeSpiralOn2
    //
    ///////////////////////////////////////////////////////////////////////////

    public void computeSpiralOn2() throws QuadInternalErrorException
    {
        if ( m_rgVertices.size() <= 2 )
            return ;

        // Find min x vertex O(n)
        m_vminx = new Vertex2( Double.MAX_VALUE, Double.MAX_VALUE ) ;
        Vertex2 v ;
        for ( Enumeration e = m_rgVertices.elements() ; e.hasMoreElements() ; ) 
        { 
            v = (Vertex2) e.nextElement() ;
            if ( v.x < m_vminx.x || ( v.x == m_vminx.x && v.y < m_vminx.y ) )
            {
                m_vminx = v ;
            }
        }
        
        // Clone the vertex array, in order to be able to mark the vertices
        Vector rgVertices = (Vector) m_rgVertices.clone() ;
        Vector rgSpiral = new Vector() ;

        // Start spiral with minx vertex
        rgSpiral.addElement( m_vminx ) ;
        rgVertices.removeElement( m_vminx ) ;
        double anglecur ;
        //System.out.println( " ==============================================================================" ) ;
        while ( rgVertices.size() > 0 )
        {
            if ( rgSpiral.size() < 2 )
            {
                anglecur = Math.PI/2 ;
            }
            else
            {
                // Compute angle of last two vertices in spiral chain
                Vertex2 last = (Vertex2) rgSpiral.lastElement() ;
                Vertex2 last2 = (Vertex2) rgSpiral.elementAt( rgSpiral.size()-2 ) ;

                anglecur = Math.atan( -( last.y - last2.y ) / ( last.x - last2.x ) ) ;
                if ( last.x < last2.x )
                    anglecur += Math.PI ;
                // anglecur is in [-pi/2,+3pi/2]
            }

            //System.out.println() ;
            //System.out.println( "Anglecur = " + anglecur/Math.PI + " pi" ) ;

            // Jarvis march step
            //
            // For each of the other vertices, wrap angle around to find next
            // CH vertex  O(n)
            Vertex2 vcur = (Vertex2) rgSpiral.lastElement() ;
            double anglemin = anglecur + 3*Math.PI/2 + 0.0001 ; // REVIEW
            Vertex2 vnext = null ;
            for ( Enumeration e = rgVertices.elements() ; e.hasMoreElements() ; ) 
            { 
                v = (Vertex2) e.nextElement() ;

                // Compute wrap angle
                double angle = Math.atan( -( v.y - vcur.y ) / ( v.x - vcur.x ) ) ;
                if ( v.x < vcur.x )
                    angle += Math.PI ;
                // angle is in [-pi/2,+3pi/2]
                //System.out.print( "   CorrectedAngle" + angle/Math.PI + " pi" ) ;
                
                angle = anglecur - angle ;
                // Make sure difference is positive
                if ( angle < 0 )
                    angle += 2*Math.PI ;

                //System.out.println( "   Difference" + angle/Math.PI + " pi" ) ;

                if ( angle < anglemin )
                {
                    anglemin = angle ;
                    vnext = v ;
                }
            }

            if ( vnext == null )
                throw new QuadInternalErrorException() ;

            rgVertices.removeElement( vnext ) ;
            rgSpiral.addElement( vnext ) ;
        }

        ///////////////////////////////////////////////////////////////////////
        // Add vertices to spiral

        if ( rgSpiral.size() < 2 )
            throw new QuadInternalErrorException() ;
        
        Enumeration e = rgSpiral.elements() ;
        Vertex2 v1 = (Vertex2) e.nextElement() ;
        Vertex2 v2 ;
        while ( e.hasMoreElements() ) 
        { 
            v2 = (Vertex2) e.nextElement() ;
            
            m_rgSpiral.addElement( new Edge( v1, v2 ) ) ;

            v1 = v2 ;
        }        
    }

    ///////////////////////////////////////////////////////////////////////////
    //
    // computeCloseChain
    //
    ///////////////////////////////////////////////////////////////////////////

    public void computeCloseChain() throws QuadInternalErrorException
    {
        if ( m_rgVertices.size() <= 2 )
            return ;

        // Start spiral with minx vertex
        double anglecur = Math.PI/2 ;

        // Jarvis march step
        //
        // For each of the other vertices, wrap angle around to find vertex in O(n)
        double anglemax = anglecur - Math.PI/2 - 0.0001 ;// REVIEW
        m_vPh = null ;
        int ii ;
        Enumeration e ;
        for ( ii = 0, e = m_rgVertices.elements() ; e.hasMoreElements() ; ii++ )
        { 
            Vertex2 v = (Vertex2) e.nextElement() ;
            
            // Compute wrap angle
            double angle = Math.atan( -( v.y - m_vminx.y ) / ( v.x - m_vminx.x ) ) ;
            if ( v.x < m_vminx.x )
                angle += Math.PI ;
            // angle is in [-pi/2,+3pi/2]
            //System.out.print( "   CorrectedAngle" + angle/Math.PI + " pi" ) ;
            
            angle = anglecur - angle ;
            // Make sure difference is positive
            if ( angle < 0 )
                angle += 2*Math.PI ;
            
            //System.out.println( "   Difference" + angle/Math.PI + " pi" ) ;
            
            if ( angle > anglemax )
            {
                anglemax = angle ;
                m_vPh = v ;
            }
        }
        
        if ( m_vPh == null )
            throw new QuadInternalErrorException() ;

        m_rgCloseChain.addElement( new Edge( m_vminx, m_vPh ) ) ;
    }

    ///////////////////////////////////////////////////////////////////////////
    //
    // computeInnerDiagonals
    //
    ///////////////////////////////////////////////////////////////////////////

    public void computeInnerDiagonals() throws QuadInternalErrorException, VertexException
    {
        if ( m_rgVertices.size() <= 2 )
            return ;

        // Create long long long long long final line segment (extend)
        int FARAWAYMULT = 100 ;
        Edge elast = (Edge) m_rgSpiral.lastElement() ;
        if ( elast != null ) 
            elast = (Edge) elast.clone() ;
        else
            return ;

        Vertex2 vfaraway = 
            Vertex2.add( Vertex2.mult( elast.v1, 1-FARAWAYMULT ), Vertex2.mult( elast.v2, FARAWAYMULT ) ) ;
        //System.out.println( "Faraway: "+elast.v1.x+"  "+elast.v1.y+" "+elast.v2.x+" "+elast.v2.y+
        //"  FAR: "+vfaraway.x+"  "+vfaraway.y ) ;
        elast.v2 = vfaraway ;

        // Find the line segment which intersects with the extended line segment
        int ii ;
        for ( ii = m_rgSpiral.size()-3 ; ii >= 0 ; ii-- )
        {
            Edge ecomp = (Edge) m_rgSpiral.elementAt( ii ) ;

            if ( Edge.intersectSegments( elast, ecomp ) == true )
                break ;
        }

        elast = (Edge) m_rgSpiral.lastElement() ;
        //Edge eclosing = null ;
        if ( ii < 0 )
        {
            //System.out.println( "Intersecting vertex not found." ) ;

            // Vertices are all on convex hull
            Edge efirst = (Edge) m_rgSpiral.elementAt( 0 ) ;
            m_vY = efirst.v2 ;
        }
        else
        {
            //System.out.println( "Intersecting vertex found: "+ii ) ;
            // Some vertices are inside the convex hull, normal case
            // Search for last vertex parallel to last segment (using rotating calipers)

            double prevdist = -1 ;
            Edge edg = null ;
            for ( double dist = 0 ; dist > prevdist ; ii--, prevdist = dist )
            {            
                edg = (Edge) m_rgSpiral.elementAt( ii ) ;
                dist = Vertex2.distance( elast.v1, elast.v2, edg.v1 ) ;
            }

            // Add the vertex to close the inner star-shaped region.
            m_vY = edg.v1 ;
        }

        // Triangulate the star-shaped polygon
        int iii = m_rgSpiral.size()-2 ;
        Edge edg ;
        do
        {
            edg = (Edge) m_rgSpiral.elementAt( iii-- ) ; 
            m_rgInnerDiagonals.addElement( new Edge( elast.v2, edg.v1 ) ) ;
        }
        while ( edg.v1 != m_vY ) ;

//         // Add closing edge (in order) to close the region if needed
//         if ( eclosing != null )
//         {
//             m_rgInnerDiagonals.addElement( eclosing ) ;
//         }
    }

    ///////////////////////////////////////////////////////////////////////////
    //
    // computeChainDiagonals
    //
    ///////////////////////////////////////////////////////////////////////////

    public void computeChainDiagonals() throws QuadInternalErrorException
    {
        if ( m_rgVertices.size() <= 2 )
            return ;
        if ( m_vY == null )
            return ;
            

        // Create polygonal annulusses
        Vector rgAnnulusOuter = new Vector() ;
        Vector rgAnnulusInner = new Vector() ;

        ///////////////////////////////////////////////////////////////////////
        // Create the outer annulus [p_1, ... , Y]

        Enumeration e = m_rgSpiral.elements(); 
        Edge edg ;
        do
        {
            edg = (Edge) e.nextElement() ; 
            rgAnnulusOuter.addElement( edg.v1 ) ;
        }
        while ( edg.v1 != m_vY && e.hasMoreElements() ) ;

        ///////////////////////////////////////////////////////////////////////
        // Create the inner annulus [p_h, ... , p_n]

        // Skip vertices before Ph
        e = m_rgSpiral.elements() ;
        Vertex2 vv ;
        do 
        {
            edg = (Edge) e.nextElement() ;
            vv = edg.v2 ;
        }
        while ( vv != m_vPh && e.hasMoreElements() ) ;

        rgAnnulusInner.addElement( m_vPh ) ;
        // Add all the remaining edges to the form the inner annulus
        while ( e.hasMoreElements() )
        {
            Edge edg1 = (Edge) e.nextElement() ;
            rgAnnulusInner.addElement( edg1.v2 ) ;
        }

//         for ( Enumeration e = rgAnnulusOuter.elements() ; e.hasMoreElements() ; )
//         {
//             Vertex2 v = (Vertex2) e.nextElement() ;
//             m_rgChainDiagonals.addElement( new Edge( new Vertex2( 100, 0 ), v ) ) ;
//         }
//         for ( Enumeration e = rgAnnulusInner.elements() ; e.hasMoreElements() ; )
//         {
//             Vertex2 v = (Vertex2) e.nextElement() ;
//             m_rgChainDiagonals.addElement( new Edge( new Vertex2( 0, 0 ), v ) ) ;
//         }

        ///////////////////////////////////////////////////////////////////////
        // Run Spiraling Rotating Calipers algorithm on polygonal annulusses
        //
        Vertex2 vouter = (Vertex2) rgAnnulusOuter.elementAt( 0 ) ;
        Vertex2 vinner = (Vertex2) rgAnnulusInner.elementAt( 0 ) ;
        double angleSRC = Vertex2.anglePoints( vinner, vouter ) ;
        double angleouter ;
        double angleinner ;
        int iouter = 0 ;
        int iinner = 0 ;
        Vertex2 vnextouter = null ;
        Vertex2 vnextinner = null ;
        // Compute angle of next two vertices in annulusses
        while ( iinner != rgAnnulusInner.size()-1 || iouter != rgAnnulusOuter.size()-1 )
        {
            ///////////////////////////////////////////////////////////////////
            // Special case 1: Simply complete advance on outer chain

            if ( iinner == rgAnnulusInner.size()-1 )
            {
                vnextouter = (Vertex2) rgAnnulusOuter.elementAt( iouter+1 ) ;
                m_rgChainDiagonals.addElement( new Edge( vinner, vnextouter ) ) ;
                iouter++ ;
                vouter = vnextouter ;
                continue ;
            }

            ///////////////////////////////////////////////////////////////////
            // Special case 1: Simply complete advance on inner chain

            if ( iouter == rgAnnulusOuter.size()-1 )
            {
                vnextinner = (Vertex2) rgAnnulusInner.elementAt( iinner+1 ) ;
                m_rgChainDiagonals.addElement( new Edge( vnextinner, vouter ) ) ;
                iinner++ ;
                vinner = vnextinner ;
                continue ;
            }

            ///////////////////////////////////////////////////////////////////
            // Normal case

            // Compute angles for next segments on polygonal chains
            vnextinner = (Vertex2) rgAnnulusInner.elementAt( iinner+1 ) ;
            vnextouter = (Vertex2) rgAnnulusOuter.elementAt( iouter+1 ) ;

            angleinner = Vertex2.anglePoints( vinner, vnextinner ) ;
            angleouter = Vertex2.anglePoints( vouter, vnextouter ) ;
            
            // Compute differences between angles and current SRC angle
            double diffouter = angleSRC - angleouter ;
            if ( diffouter < 0 ) diffouter += 2*Math.PI ;
            double diffinner = angleSRC - angleinner ;
            if ( diffinner < 0 ) diffinner += 2*Math.PI ;
        
            // Choose the caliper side with the smallest difference
            if ( diffinner < diffouter )
            {
                // Inner was hit by caliper first
                m_rgChainDiagonals.addElement( new Edge( vnextinner, vouter ) ) ;
                iinner++ ;
                vinner = vnextinner ;
                angleSRC = angleinner ;
            }
            else
            {
                // Outer was hit by caliper first
                m_rgChainDiagonals.addElement( new Edge( vinner, vnextouter ) ) ;
                iouter++ ;
                vouter = vnextouter ;
                angleSRC = angleouter ;
            }
        }
        
        // Remove last element, it was already inserted in the inner
        // star-shaped chain
        m_rgChainDiagonals.removeElementAt( m_rgChainDiagonals.size()-1 ) ;
    }

    ///////////////////////////////////////////////////////////////////////////
    //
    // computeRemoveDiagonals
    //
    ///////////////////////////////////////////////////////////////////////////

    public void computeRemoveDiagonals() throws QuadInternalErrorException
    {
        if ( m_rgVertices.size() <= 2 )
            return ;

        int correct = 0 ;
        if ( m_rgInnerDiagonals.size() % 2 == 1 )
            correct = 1 ;

        ///////////////////////////////////////////////////////////////////////
        // Start removing one out of two diagonals from inner star-shaped region
        //
        Vector rgNewInnerDiagonals = new Vector() ;
        for ( int ii = 0 ; ii < m_rgInnerDiagonals.size()-1 ; ii += 2 )
        {
            rgNewInnerDiagonals.addElement( m_rgInnerDiagonals.elementAt( ii+1 ) ) ;
        }
        m_rgInnerDiagonals = rgNewInnerDiagonals ;

        ///////////////////////////////////////////////////////////////////////
        // Start removing one out of two diagonals from outer star-shaped
        // poylgonal annulus
        //

        Vector rgNewChainDiagonals = new Vector() ; 
        for ( int ii = m_rgChainDiagonals.size()-1+correct ; ii >= 1 ; ii -= 2 )
        {
            rgNewChainDiagonals.addElement( m_rgChainDiagonals.elementAt( ii-1 ) ) ;
        }
        m_rgChainDiagonals = rgNewChainDiagonals ;
    }

    ///////////////////////////////////////////////////////////////////////////
    //
    // computeSteiner
    //
    ///////////////////////////////////////////////////////////////////////////

    public void computeSteiner() throws QuadInternalErrorException
    {
        if ( m_rgVertices.size() <= 2 )
            return ;
        if ( m_vPh == null )
            return ;

        // Count the number of vertices on the convex hull [P1, ..., Ph] incl.
        Enumeration e = m_rgSpiral.elements() ;
        Edge edg ;
        int inbverts = 0 ;
        do
        {
            edg = (Edge) e.nextElement() ;
            inbverts++ ;
        }
        while ( edg.v2 != m_vPh ) ;
        inbverts++ ;

        //System.out.println( "CH vertices: " + inbverts ) ;

        if ( inbverts % 2 == 1 )
        {
            // The number of vertices on the CH is odd, add a Steiner point
            edg = (Edge) m_rgCloseChain.elementAt( 0 ) ;
            Vertex2 vSteiner = new Vertex2( (edg.v1.x + edg.v2.x)/2,
                                            (edg.v1.y + edg.v2.y)/2 ) ;
            m_rgVertices.addElement( vSteiner ) ;

            // Change the closing chain to include the Steiner point
            m_rgCloseChain.removeAllElements() ;
            m_rgCloseChain.addElement( new Edge( edg.v1, vSteiner ) ) ;
            m_rgCloseChain.addElement( new Edge( vSteiner, edg.v2 ) ) ;
        }
    }


    ///////////////////////////////////////////////////////////////////////////
    //
    // computeFinalQuadrang
    //
    ///////////////////////////////////////////////////////////////////////////

    public void computeFinalQuadrang() throws QuadInternalErrorException
    {
        if ( m_rgVertices.size() <= 2 )
            return ;
        m_rgFinalQuadrang.removeAllElements() ;

        // Simply put all the edges indiscriminately in a quadrangulation
        Edge edg ;
        for ( Enumeration e = m_rgSpiral.elements() ; e.hasMoreElements() ; )
        {
            edg = (Edge) e.nextElement() ;
            m_rgFinalQuadrang.addElement( edg ) ;
        }
        for ( Enumeration e = m_rgCloseChain.elements() ; e.hasMoreElements() ; )
        {
            edg = (Edge) e.nextElement() ;
            m_rgFinalQuadrang.addElement( edg ) ;
        }
        for ( Enumeration e = m_rgInnerDiagonals.elements() ; e.hasMoreElements() ; )
        {
            edg = (Edge) e.nextElement() ;
            m_rgFinalQuadrang.addElement( edg ) ;
        }
        for ( Enumeration e = m_rgChainDiagonals.elements() ; e.hasMoreElements() ; )
        {
            edg = (Edge) e.nextElement() ;
            m_rgFinalQuadrang.addElement( edg ) ;
        }
    }


    ///////////////////////////////////////////////////////////////////////////
    // QuadApp.minimumSize
    //
    public Dimension minimumSize()
    {
        return new Dimension( 300, 200 ) ;
    }

    ///////////////////////////////////////////////////////////////////////////
    // QuadApp.preferredSize
    //
    public Dimension preferredSize()
    {
        return new Dimension( 800, 800 ) ;
    }

    ///////////////////////////////////////////////////////////////////////////
    // QuadApp.terminate
    //
    public void terminate()
    {
        if ( m_fInAnApplet )
        {
            dispose() ;
        }
        else
        {
            System.exit( 0 ) ;
        }
    }

    ///////////////////////////////////////////////////////////////////////////
    // QuadApp.getManualDialog
    //
    public Frame getManualDialog()
    {
        if ( m_dlgManual == null )
        {
            m_dlgManual = new ManualDialog( this, false, m_codeBase ) ;
        }
        return m_dlgManual ;
    }
    
    ///////////////////////////////////////////////////////////////////////////
    // QuadApp.getAboutDialog
    //
    public Frame getAboutDialog()
    {
        if ( m_dlgAbout == null )
        {
            m_dlgAbout = new AboutDialog( this, m_imgQuaddy, m_fInAnApplet ) ;
        }
        return m_dlgAbout ;
    }

    ///////////////////////////////////////////////////////////////////////////
    // QuadApp.main
    //
    public static void main( String astring[] )
    {
	// Load images
	Image imgQuaddy = Toolkit.getDefaultToolkit().getImage( "Quaddy.gif" ) ;

        QuadApp app = new QuadApp( imgQuaddy, null ) ;
        app.m_fInAnApplet = false ;
        app.pack();
        app.show();
    }
}









///////////////////////////////////////////////////////////////////////////////
//
// class QuadInternalErrorException
//
// General purpose internal error exception
//
///////////////////////////////////////////////////////////////////////////////

class QuadInternalErrorException extends Exception
{
    public QuadInternalErrorException()
    {
        super() ;
    }

    public QuadInternalErrorException( String  s )
    {
        super( s ) ;
    }
}

