/*****************************************************************************\
 * Vertex2.java
 *
 * $Author: blais $
 * $Date: 1997/10/20 01:59:16 $
 *
 * Desc:        Vertex2 class.
 *
 *****************************************************************************/
/*****************************************************************************\
 * Copyright (C) 1996, Martin Blais.
 * All Rights Reserved.
 * 
 * This is unpublished proprietary source code of Martin Blais;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Martin Blais.
 *****************************************************************************/

import java.lang.Exception ;
import java.lang.Math ;

///////////////////////////////////////////////////////////////////////////////
//
// class VertexException
//
///////////////////////////////////////////////////////////////////////////////

class VertexException extends Exception
{
    public VertexException()
    {
        super() ;
    }

    public VertexException( String  s )
    {
        super( s ) ;
    }
}


///////////////////////////////////////////////////////////////////////////////
//
// Vertex2
//
///////////////////////////////////////////////////////////////////////////////

class Vertex2 extends Object implements Cloneable
{
    /////////////////////////////////////////////////////////////////////////////
    // Data members

    public double x ;
    public double y ;

    ///////////////////////////////////////////////////////////////////////////
    // Constructors
    //
    Vertex2()
    {  x = 0 ;  y = 0 ;    }

    Vertex2( double X, double Y )
    {  x = X ; y = Y ;  }

    Vertex2( Vertex2 v )
    {  x = v.x ; y = v.y ;  }

    Vertex2( double X ) 
    {  x = X ;  y =  X ; }


    ///////////////////////////////////////////////////////////////////////////
    // Clone
    //
    public Object clone() 
    {
        return new Vertex2( this ) ;
    }

    ///////////////////////////////////////////////////////////////////////////
    // equals
    //
    public boolean equals( Object obj ) 
    {
        Vertex2 v = (Vertex2) obj ;
        return x == v.x && y == v.y ;
    }

    ///////////////////////////////////////////////////////////////////////////
    // Assignment
    //
    public void set( double nx, double ny ) 
    {
        x = nx ;  y = ny ;
    }

    ///////////////////////////////////////////////////////////////////////////
    // Norm
    //
    public double norm() 
    {
        return Math.sqrt( x*x + y*y ) ;
    }

    public static double norm( Vertex2 v ) 
    {
        return Math.sqrt( v.x*v.x + v.y*v.y ) ;
    }

    ///////////////////////////////////////////////////////////////////////////
    // Square Norm
    //
    public double sqrNorm() 
    {
        return x*x + y*y ;
    }

    public static double sqrNorm( Vertex2 v ) 
    {
        return v.x*v.x + v.y*v.y ;
    }
    
    ///////////////////////////////////////////////////////////////////////////
    // Normalization
    //
    public void normalize()
    {
        divide( norm() ) ;
    }

    public static Vertex2 normalize( Vertex2 v )
    {
        return divide( v, norm( v ) ) ;
    }

    
    ///////////////////////////////////////////////////////////////////////////
    // Addition
    //
    public 			void add( Vertex2 v )
    {
        x += v.x ;  y += v.y ;
    }

    public static 	Vertex2 add( Vertex2 v1, Vertex2 v2 )
    {
        Vertex2 v = new Vertex2( v1 ) ;
        v.add( v2 ) ;
        return v ;
    }

    ///////////////////////////////////////////////////////////////////////////
    // Substraction
    //
    public        void minus( Vertex2 v )
    {
        x -= v.x ;  y -= v.y ;
    }

    public static Vertex2 minus( Vertex2 v1, Vertex2 v2 )
    {
        Vertex2 v = new Vertex2( v1 ) ;
        v.minus( v2 ) ;
        return v ;
    }


    ///////////////////////////////////////////////////////////////////////////
    // Multiplication
    //
    public        void mult( Vertex2 v )
    {
        x *= v.x ;  y *= v.y ;
    }

    public static Vertex2 mult( Vertex2 v1, Vertex2 v2 )
    {
        Vertex2 v = new Vertex2( v1 ) ;
        v.mult( v2 ) ;
        return v ;
    }

    public        void mult( double dbl )
    {
        x *= dbl ;  y *= dbl ;
    }

    public static Vertex2 mult( Vertex2 v1, double dbl )
    {
        Vertex2 v = new Vertex2( v1 ) ;
        v.mult( dbl ) ;
        return v ;
    }

    public static Vertex2 mult( double dbl, Vertex2 v )
    {
        return mult( v, dbl ) ;
    }

    ///////////////////////////////////////////////////////////////////////////
    // Division
    //
    public        void  divide( Vertex2 v )
    {
        x /= v.x ;  y /= v.y ;
    }

    public static Vertex2 divide( Vertex2 v1, Vertex2 v2 )
    {
        Vertex2 v = new Vertex2( v1 ) ;
        v.divide( v2 ) ;
        return v ;
    }

    public        void divide( double dbl )
    {
        x /= dbl ;  y /= dbl ;
    }

    public static Vertex2 divide( Vertex2 v1, double dbl )
    {
        Vertex2 v = new Vertex2( v1 ) ;
        v.divide( dbl ) ;
        return v ;
    }


    ///////////////////////////////////////////////////////////////////////////
    // Dot product
    //
    public static double dotProd( Vertex2 v1, Vertex2 v2 )
    {
        return v1.x * v2.x + v1.y * v2.y ;
    }

    ///////////////////////////////////////////////////////////////////////////
    // Cross product
    //
    public static Vertex2 crossProd ( Vertex2 v1, Vertex2 v2 )
    {
        return new Vertex2( v1.x * v2.y - v1.y * v2.x ) ;
    }

    ///////////////////////////////////////////////////////////////////////////
    // Compute cosine between two vertex
    public static double computeCos( Vertex2 v1, Vertex2 v2 )
    {
        double dblMag = norm( v1 ) * norm( v2 ) ;
        if ( dblMag == 0 )
        {
            return Double.MAX_VALUE ;
        }
        return dotProd( v1, v2 ) / dblMag ;
    }

    ///////////////////////////////////////////////////////////////////////////
    // Minimize
    //
    public static void minimize( Vertex2 min, Vertex2 candidate )
    {
        if ( candidate.x < min.x )
        {
            min.x = candidate.x ;
        }
        if ( candidate.y < min.y )
        {
            min.y = candidate.y ;
        }
    }

    ///////////////////////////////////////////////////////////////////////////
    // Maximize
    //
    public static void maximize( Vertex2 max, Vertex2 candidate )
    {
        if ( candidate.x > max.x )
        {
            max.x = candidate.x ;
        }
        if ( candidate.y > max.y )
        {
            max.y = candidate.y ;
        }
    }

    ///////////////////////////////////////////////////////////////////////////
    // Signed area of a triangle
    //
    public static double signedArea( Vertex2 v1, Vertex2 v2, Vertex2 v3 )
        throws VertexException
    {
        if ( v1.equals( v2 ) || v2.equals( v3 ) || v3.equals( v1 ) )
        {
            throw new VertexException() ;
        }

        return ( v1.x * ( v2.y - v3.y ) + v2.x * ( v3.y - v1.y ) + v3.x * ( v1.y - v2.y ) ) / 2.0 ; 
    }


    ///////////////////////////////////////////////////////////////////////////
    // Perpendicular distance from an infinite line to a point
    //
    // Note: Distance is positive if the point is to the left of the line.  
    // This convention has been chosen so that it is convenient to think of it
    // as the right-hand rule.
    //
    public static double distance( Vertex2 vLine1, Vertex2 vLine2, Vertex2 vPoint )
        throws VertexException
    {
        // b*h/2 = area... h = area*2/b
        Vertex2 vline = minus( vLine2, vLine1 ) ;
        return signedArea( vLine1, vLine2, vPoint ) / norm( vline ) ;

//         Vertex2 vldir = minus( vLine2, vLine1 ) ;
//         Vertex2 vpdir = minus( vPoint, vLine1 ) ;
//         double dp = dotProd( vldir, vpdir ) / magnitude( vldir ) ;
//         Vertex2 vint = add( v1, mult( vldir, dp ) ) ;
//         Vertex2 vdist = minus( vPoint, vint ) ;
//         return magnitude( 
    }

    ///////////////////////////////////////////////////////////////////////////
    // Angle determined by points v1 and v2
    //
    // Note: Returns an angle between [-pi/2,+3pi/2]
    //
    public static double anglePoints( Vertex2 v1, Vertex2 v2 ) 
    {
        double angle = Math.atan( -( v2.y - v1.y ) / ( v2.x - v1.x ) ) ;
        if ( v2.x < v1.x )
            angle += Math.PI ;
        return angle ;
    }

}



