/*****************************************************************************\
 * Vertex3.java
 *
 * $Author: blais $
 * $Date: 1997/10/12 07:26:08 $
 *
 * Desc:        Vertex3 class.
 *
 *****************************************************************************/
/*****************************************************************************\
 * Copyright (C) 1996, Martin Blais.
 * All Rights Reserved.
 * 
 * This is unpublished proprietary source code of Martin Blais;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Martin Blais.
 *****************************************************************************/

///////////////////////////////////////////////////////////////////////////////
//
// Vertex3
//
///////////////////////////////////////////////////////////////////////////////

class Vertex3
{
    /////////////////////////////////////////////////////////////////////////////
    // Data members

    public double x ;
    public double y ;
    public double z ;

    ///////////////////////////////////////////////////////////////////////////
    // Constructors
    //
    Vertex3()
    {  x = 0 ;  y = 0 ;  z = 0 ;  }

    Vertex3( double X, double Y, double Z )
    {  x = X ; y = Y ; z = Z ; }

    Vertex3( Vertex3 v )
    {  x = v.x ; y = v.y ; z = v.z ; }

    Vertex3( double X ) 
    {  x = y = z = X ; }

    ///////////////////////////////////////////////////////////////////////////
    // Assignment
    //
    public void set( double nx, double ny, double nz ) 
    {
        x = nx ;  y = ny ;  z = nz ;
    }

    ///////////////////////////////////////////////////////////////////////////
    // Norm
    //
    public double norm() 
    {
        return Math.sqrt( x*x + y*y + z*z ) ;
    }

    public static double norm( Vertex3 v ) 
    {
        return Math.sqrt( v.x*v.x + v.y*v.y + v.z*v.z ) ;
    }

    ///////////////////////////////////////////////////////////////////////////
    // Square Norm
    //
    public double sqrNorm() 
    {
        return x*x + y*y + z*z ;
    }

    public static double sqrNorm( Vertex3 v ) 
    {
        return v.x*v.x + v.y*v.y + v.z*v.z ;
    }
    
    ///////////////////////////////////////////////////////////////////////////
    // Normalization
    //
    public void normalize()
    {
        divide( norm() ) ;
    }

    public static Vertex3 normalize( Vertex3 v )
    {
        return divide( v, norm( v ) ) ;
    }

    
    ///////////////////////////////////////////////////////////////////////////
    // Addition
    //
    public 			void add( Vertex3 v )
    {
        x += v.x ;  y += v.y ; z += v.z ;
    }

    public static 	Vertex3 add( Vertex3 v1, Vertex3 v2 )
    {
        Vertex3 v = new Vertex3( v1 ) ;
        v.add( v2 ) ;
        return v ;
    }

    ///////////////////////////////////////////////////////////////////////////
    // Substraction
    //
    public        void minus( Vertex3 v )
    {
        x -= v.x ;  y -= v.y ; z -= v.z ;
    }

    public static Vertex3 minus( Vertex3 v1, Vertex3 v2 )
    {
        Vertex3 v = new Vertex3( v1 ) ;
        v.minus( v2 ) ;
        return v ;
    }


    ///////////////////////////////////////////////////////////////////////////
    // Multiplication
    //
    public        void mult( Vertex3 v )
    {
        x *= v.x ;  y *= v.y ; z *= v.z ;
    }

    public static Vertex3 mult( Vertex3 v1, Vertex3 v2 )
    {
        Vertex3 v = new Vertex3( v1 ) ;
        v.mult( v2 ) ;
        return v ;
    }

    public        void mult( double dbl )
    {
        x *= dbl ;  y *= dbl ; z *= dbl ;
    }

    public static Vertex3 mult( Vertex3 v1, double dbl )
    {
        Vertex3 v = new Vertex3( v1 ) ;
        v.mult( dbl ) ;
        return v ;
    }

    public static Vertex3 mult( double dbl, Vertex3 v )
    {
        return mult( v, dbl ) ;
    }

    ///////////////////////////////////////////////////////////////////////////
    // Division
    //
    public        void  divide( Vertex3 v )
    {
        x /= v.x ;  y /= v.y ; z /= v.z ;
    }

    public static Vertex3 divide( Vertex3 v1, Vertex3 v2 )
    {
        Vertex3 v = new Vertex3( v1 ) ;
        v.divide( v2 ) ;
        return v ;
    }

    public        void divide( double dbl )
    {
        x /= dbl ;  y /= dbl ; z /= dbl ;
    }

    public static Vertex3 divide( Vertex3 v1, double dbl )
    {
        Vertex3 v = new Vertex3( v1 ) ;
        v.divide( dbl ) ;
        return v ;
    }


    ///////////////////////////////////////////////////////////////////////////
    // Dot product
    //
    public static double dotProd( Vertex3 v1, Vertex3 v2 )
    {
        return v1.x * v2.x + v1.y * v2.y + v1.z * v2.z ;
    }

    ///////////////////////////////////////////////////////////////////////////
    // Cross product
    //
    public static Vertex3 crossProd ( Vertex3 v1, Vertex3 v2 )
    {
        return new Vertex3( v1.y * v2.z - v1.z * v2.y,
                           v1.z * v2.x - v1.x * v2.z,
                           v1.x * v2.y - v1.y * v2.x ) ;
    }

    ///////////////////////////////////////////////////////////////////////////
    // Compute cosine between two vertex
    public static double computeCos( Vertex3 v1, Vertex3 v2 )
    {
        double dblMag = norm( v1 ) * norm( v2 ) ;
        if ( dblMag == 0 )
        {
            return Double.MAX_VALUE ;
        }
        return dotProd( v1, v2 ) / dblMag ;
    }

    ///////////////////////////////////////////////////////////////////////////
    // Minimize
    //
    public static void minimize( Vertex3 min, Vertex3 candidate )
    {
        if ( candidate.x < min.x )
        {
            min.x = candidate.x ;
        }
        if ( candidate.y < min.y )
        {
            min.y = candidate.y ;
        }
        if ( candidate.z < min.z )
        {
            min.z = candidate.z ;
        }
    }

    ///////////////////////////////////////////////////////////////////////////
    // Maximize
    //
    public static void maximize( Vertex3 max, Vertex3 candidate )
    {
        if ( candidate.x > max.x )
        {
            max.x = candidate.x ;
        }
        if ( candidate.y > max.y )
        {
            max.y = candidate.y ;
        }
        if ( candidate.z > max.z )
        {
            max.z = candidate.z ;
        }
    }
}

